import math

import numpy as np

from .. import devo



    # fitness function

def pre_perfs(perfs):
    try:
        perfs = perfs['fitness']
    except Exception:
        pass
    return perfs

def fitness_fun(window_min, window_max, n=10):
    def _fitness_aux(perfs):
        perfs = pre_perfs(perfs)
        return np.mean(np.sort(perfs[window_min:window_max].flatten())[-n:])
    return _fitness_aux

def fitness_3950_4000(perfs):
    perfs = pre_perfs(perfs)
    return np.mean(np.sort(perfs[3950:4000].flatten())[-10:])

def fitness_0_4000(perfs):
    perfs = pre_perfs(perfs)
    return np.mean(np.sort(perfs[0:4000].flatten())[-10:])

def fitness_2500_diff_4000(perfs):
    perfs = pre_perfs(perfs)
    return np.mean(np.sort(perfs[3950:4000].flatten())[-10:]) - np.mean(np.sort(perfs[2500:2550].flatten())[-10:])

def fitness_500_diff_1000(perfs):
    perfs = pre_perfs(perfs)
    return np.mean(np.sort(perfs[950:1000].flatten())[-10:]) - np.mean(np.sort(perfs[500:550].flatten())[-10:])


    # x function

def counter(start=1):
    def aux(cfg, count={'count': start-1}):
        count['count'] += 1
        return count['count']
    return aux

def double_counter():
    def aux(cfg, count={'count': 0}):
        count['count'] += 1
        return (count['count'] + 1)// 2
    return aux

def one_delay_triple_counter():
    def aux(cfg, count={'count': 0}):
        if count['count'] == 0:
            count['count'] += 2
            return 1
        else:
            count['count'] += 1
            return (count['count'] + 3)// 3
    return aux


def multi_counter(n):
    def aux(cfg, count={'count': 0}):
        count['count'] += 1
        return (count['count'] + 1)// n
    return aux

def minor_x(cfg):
    return cfg['meta']['minor']



def birth_stiffness_x(cfg):
    if 'muscle_stiff' in cfg['population']['dev_vars']:
        return cfg['population']['dev_vars']['muscle_stiff']['milestones'][0][1]/20000
    else:
        return cfg['member']['materials']['link_sec_side']['stiffness']/20000

def stiffness_x(cfg):
    if 'muscle_stiff' in cfg['population']['dev_vars']:
        return cfg['population']['dev_vars']['muscle_stiff']['milestones'][-1][1]/20000
    else:
        return cfg['member']['materials']['link_sec_side']['stiffness']/20000

def stiffness_indexed_x(cfg):
    if 'muscle_stiff' in cfg['population']['dev_vars']:
        return devo.TrajectoryGenerator(cfg['population']['dev_vars']['muscle_stiff'])(0)/20000
    else:
        return cfg['member']['materials']['link_sec_side']['stiffness']/20000

def stiffness_index_x(cfg):
    if 'muscle_stiff' in cfg['population']['dev_vars']:
        return cfg['population']['dev_vars']['muscle_stiff']['index']['milestones'][0][1]
    else:
        return 1.0

def mass_x(cfg):
    if 'node_mass' in cfg['population']['dev_vars']:
        return cfg['population']['dev_vars']['node_mass']['milestones'][0][1]
    else:
        return cfg['member']['materials']['node_section']['mass']

def mass_indexed_x(cfg):
    if 'node_mass' in cfg['population']['dev_vars']:
        return devo.TrajectoryGenerator(cfg['population']['dev_vars']['node_mass'])(0)
    else:
        return cfg['member']['materials']['node_section']['mass']

def mass_index_x(cfg):
    if 'node_mass' in cfg['population']['dev_vars']:
        return cfg['population']['dev_vars']['node_mass']['index']['milestones'][0][1]
    else:
        return 1.0

def birthsize_x(cfg):
    if 'birth_size' in cfg['population']['dev_vars']:
        return cfg['population']['dev_vars']['birth_size']['milestones'][0][1]
    else:
        return cfg['member']['birth_size']

def evodevo_birthsize_x(cfg):
    if 'birth_size' in cfg['member']['dev_vars']:
        return cfg['member']['dev_vars']['birth_size']['milestones'][0][1]
    else:
        return cfg['member']['birth_size']

def sizemass_x(cfg):
    if 'birth_size' in cfg['population']['dev_vars']:
        return cfg['population']['dev_vars']['birth_size']['milestones'][0][1]
    if 'node_mass' in cfg['population']['dev_vars']:
        return cfg['population']['dev_vars']['node_mass']['milestones'][0][1]
    else:
        return cfg['member']['birth_size']

def devlength_size_x(cfg):
    if 'dev_vars' in cfg['population'] and len(cfg['population']['dev_vars']) > 0:
        assert 'birth_size' in cfg['population']['dev_vars']
        return cfg['population']['dev_vars']['birth_size']['milestones'][-1][0]
    else:
        return 0

def devlength_mass_index_x(cfg):
    if 'dev_vars' in cfg['population'] and len(cfg['population']['dev_vars']) > 0:
        assert 'node_mass' in cfg['population']['dev_vars']
        return cfg['population']['dev_vars']['node_mass']['index']['milestones'][-1][0]
    else:
        return 0

def gravity_x(cfg):
    if 'gravity' in cfg['population']['dev_vars']:
        return -cfg['population']['dev_vars']['gravity']['milestones'][0][1]
    else:
        return -cfg['member']['environment']['gravity']

def gravity_indexed_x(cfg):
    if 'gravity' in cfg['population']['dev_vars']:
        return devo.TrajectoryGenerator(cfg['population']['dev_vars']['gravity'])(0)
    else:
        return cfg['member']['environment']['gravity']


def population_x(cfg):
    return cfg['population']['n_mutant']

def noise_x(cfg):
    motor_noise = cfg['member']['motor_noise']
    if motor_noise == 0.0:
        return -10
    return math.log10(motor_noise)

def n_section_x(cfg):
    return cfg['member']['n_section']
