import os
import numbers
import numpy as np

from ... import devo, paths
from ..  import loadlib
from .   import graph



color_list = ['#332288', '#88ccee', '#44aa99', '#117733', '#999933', '#ddcc77']
#color_list = ['#045a8d', '#2b8cbe', '#74a9cf', '#a6bddb', '#d0d1e6', '#f1eef6']

def save(exp_name, x, Y, filename='', y_range=None, width=600):
    g = graph.Graph(title=filename, width=600, height=500, y_range=y_range) #, x_range=(0, 11))
    if np.min(Y) >= 0:
        g.fig.y_range.start = 0.0
    # else:
    #     g.fig.y_range.end   = 0.0
    g.set_x_ticks([0, 1000, 2000, 3000, 4000])

    for i, y in enumerate(Y):
        g.set_y_ticks(sorted(set([0.0, min(y), y[3999]])))
        # g.set_y_ticks(sorted(set([0.0, y[0], y[1000], y[2000], y[3000], y[3999]])))
        g.fig.axis.major_label_text_font = 'avenir next'
        g.fig.axis.major_label_text_font_size = '20pt'
        # g.fig.line(x, y, alpha=0.75, line_color=color_list[i + 1], line_width=4)
        # g.fig.scatter([0, 1000, 2000, 3999], [y[0], y[1000], y[2000], y[3999]], alpha=0.75,
        #               fill_color=color_list[1], line_color=None, size=10)

        g.fig.line(x, y, alpha=0.75, line_color=color_list[i + 1], line_width=1)
        g.fig.scatter([0, 2000, 3999], [y[0], y[2000], y[3999]], alpha=0.75,
                      fill_color=color_list[1], line_color=None, size=5)


    fig_filepath = os.path.join(paths.FIGURES_DIR, exp_name, 'devgraphs/{}.pdf'.format(filename))
    graph.save_fig(g.fig, fig_filepath, scale_factor=1.0)
    g.show()

def splice(Y):
    if isinstance(Y[0], numbers.Number):
        return [Y]
    else:
        return list(zip(*Y))

def dev_traj(exp_name, dev_cfg, name_prefix, T=4000, y_range=None, **kwargs):
    for dev_name, dev_params in dev_cfg['dev_vars'].items():
        dev_gen = devo.TrajectoryGenerator(dev_params)
        x = range(T)
        Y = splice([dev_gen(t) for t in x])
        save(exp_name, x, Y, name_prefix + '_' + dev_name, y_range=y_range, **kwargs)

def dev_graph(exp_name, major, minor, **kwargs):
    cfg = loadlib.load_cfg(exp_name, major, minor)
    dev_cfg = cfg['population']
    name_prefix = '{}_{}'.format(major, minor)
    traj = dev_traj(exp_name, dev_cfg, name_prefix, **kwargs)
