import os
import pickle
from pathlib import Path

import numpy as np

from ... import paths
from ... import utils
from ..  import loadlib
from .   import graph
from .   import color


def elite_fitness_graph(params, major_a, minor_a, major_b, minor_b, pool=None):
    exp_name = params['exp_name']
    x_range, y_range, color_names = params.get('x_range', None), params['y_range'], params['color_names']
    x_ticker, x_eval = params.get('x_ticker', None), params.get('x_eval', False)
    fitness_function = params.get('fitness_function', None)
    width, height = params.get('fig_size', (800, 300))
    font_size = params.get('font_size', '25pt')

    version = '_'.join(['v{}'.format(loadlib.version_str(major, minor))
                                for major, minor in [(major_a, minor_a), (major_b, minor_b)]])
    x_eval_str = '_x_eval' if x_eval else ''
    filename = 'elite_fitness_{}{}_c{}'.format(version, x_eval_str, params['confidence'])
    fig_filepath = Path(paths.FIGURES_DIR, exp_name, 'elite_fitness', filename)
    fig_filepath.parent.mkdir(parents=True, exist_ok=True)
    title = filename.replace('_', ' ')

    g = graph.Graph(title=title, y_range=y_range, x_range=x_range, width=width, height=height,
                    tools=None, y_axis_location='right')
    if x_ticker is not None:
        g.fig.xaxis.ticker = x_ticker
    g.fig.axis.major_label_text_font_size = font_size
    g.fig.axis.axis_line_width = 3
    g.fig.axis.major_tick_line_width = 3
    g.fig.axis.major_tick_out = 15
    g.fig.axis.axis_label_text_baseline = 'bottom'
    g.fig.axis.major_label_standoff = 20
    g.fig.min_border_right = 100

    y_ticker = [y_range[0], y_range[1]]
    for (major, minor), color_name in zip([(major_b, minor_b), (major_a, minor_a)], reversed(color_names)):
        line_color, dark_color, light_color = getattr(color, color_name)

        cfg = loadlib.load_cfg(exp_name, major, minor)
        if cfg['meta'].get('disabled', False):
            utils.print_warning('v{}.{} v{}.{} disabled, skipping'.format(major_a, minor_a, major_b, minor_b))
            return

        post_filepath = loadlib.elite_fitness_filepath(exp_name, major, minor, params['confidence'], fitness_function)
        with open(post_filepath, 'rb') as fd:
            data = pickle.load(fd)

        # data = data
        factor = 1
        if params.get('x_eval', False):
            factor = cfg['population']['n_mutant'] + cfg['population']['n_elites']
        mean, mean_ci_low, mean_ci_high = data['mean'], data['mean_ci_low'], data['mean_ci_high']
        # mean, mean_ci_low, mean_ci_high = data['mean']/1000, data['mean_ci_low']/1000, data['mean_ci_high']/1000
        generations = range(factor, factor * (len(mean) + 1), factor)
        patch_x = list(generations) + list(reversed(generations))
        patch_y = list(mean_ci_low) + list(reversed(list(mean_ci_high)))
        g.fig.patch(patch_x, patch_y, line_color=None, fill_color=light_color, fill_alpha=0.5)
        g.fig.line(generations, mean, line_color=line_color, line_alpha=1.0, line_width=2.0)
        #g.fig.line([750, 750], y_range, line_color='grey', line_alpha=0.25, line_width=1.0)
        y_ticker.append(mean[-1])


    # assert y_range[0] <= min(y_ticker) and max(y_ticker) <= y_range[1]
    y_ticker = sorted([float(y_i) for y_i in y_ticker])
    g.fig.yaxis.ticker = y_ticker
    g.fig.yaxis.major_label_overrides = {y_i: '{:.2f}'.format(y_i) for y_i in y_ticker}

    return g, graph.save_fig(g.fig, str(fig_filepath) + '.png', scale_factor=1.0, verbose=True)
