import subprocess
from pathlib import Path

import numpy as np

from ... import paths
from ..  import loadlib, funlib
from .   import graph, color


def get_perfs(cfg, f_perf):

    major, minor = cfg['meta']['major'], cfg['meta']['minor']
    version = loadlib.version_str(major, minor)

    xs, ys = [], []
    for seed in range(cfg['meta']['slurm_array'][0], cfg['meta']['slurm_array'][1] + 1):
        try:
            perfs = loadlib.load_perffile(cfg, seed) / 1000
            xs.append(seed)
            ys.append(f_perf(perfs))
        except FileNotFoundError as exc:
            print('{}; skipping.'.format(exc))
    return xs, ys

def seed_comp_graph(exp_name, versions,
                    f_perf=None, title=None, filename=None, y_range=(0, 10),
                    color_names=('control', 'development'), sorted_diff=True, ext='pdf'):

    if title is None:
        title = '{}_{}'.format(exp_name, ''.join('v{}.{}'.format(major, minor) for major, minor in versions))
    if filename is None:
        filename = title
    if f_perf is None:
        f_perf = funlib.fitness_3950_4000

    X, Y = [], []
    for major, minor in versions:
        cfg = loadlib.load_cfg(exp_name, major, minor)
        xs, ys  = get_perfs(cfg, f_perf)
        X.append(xs)
        Y.append(ys)

    g = graph.Graph(title=title, x_range=(0, max(len(xs) for xs in X) + 1), y_range=y_range,
                    height=150, tools=None)
    g.fig.axis.major_label_text_font_size = '5pt'

    if sorted_diff: # TODO: handle missing data?
        assert X[0] == X[1]
        diffs = [y_1 - y_0 for y_1, y_0 in zip(Y[1], Y[0])]
        indexes = range(len(diffs))
        sorted_indexes = [i for _, i in sorted(zip(diffs, indexes))]

        sorted_seeds = [X[0][i] for i in sorted_indexes]
        g.fig.xaxis.ticker = list(range(1, len(sorted_seeds) + 1))
        g.fig.xaxis.major_label_overrides = {i+1: str(seed) for i, seed in zip(indexes, sorted_seeds)}


        X = [range(1, len(sorted_seeds) + 1) for xs in X]
        Y = [[ys[i] for i in sorted_indexes] for ys in Y]

    for i, (xs, ys) in enumerate(zip(X, Y)):
        line_color, dark_color, light_color = getattr(color, color_names[i])
        g.fig.line(xs, ys, line_color=light_color, line_alpha=1.0, line_dash=(1, 1))
        g.fig.scatter(xs, ys, fill_color=light_color, fill_alpha=1.0, line_color=None, size=5)

    # TODO: add mean

    fig_filepath = Path(paths.FIGURES_DIR, exp_name, 'seed_comp/{}.{}'.format(filename, ext))
    graph.save_fig(g.fig, fig_filepath, scale_factor=2.0)
    subprocess.check_output(['open', fig_filepath])
