import os
from pathlib import Path

from .. import loadlib



def uptodate(cfg, seeds, filepath):
    """Check that the file at filepath exists and is newer that the results' manifest file."""
    filepath = Path(filepath)
    if not filepath.is_file():
        return False

    max_mtime = float('-inf')
    for seed in seeds:
        filepath_manifest = Path(loadlib.manifest_done_path_from_cfg(cfg, seed), 'manifest.yaml')
        max_mtime = max(max_mtime, filepath_manifest.stat().st_mtime)

    return max_mtime < filepath.stat().st_mtime

def set_mtime(cfg, seeds, filepath):
    """Set the filepath's file access and modification time epsilon newer than the results' manifest file.

    The idea is to ensure that the time comparison of uptodate only occurs with file that were
    generated from this version of the manifest.
    """
    return
    assert len(seeds) > 0
    max_mtime = float('-inf')
    for seed in seeds:
        filepath_manifest = Path(loadlib.manifest_done_path_from_cfg(cfg, seed), 'manifest.yaml')
        max_mtime = max(max_mtime, filepath_manifest.stat().st_mtime)

    os.utime(filepath, (max_mtime + 0.1, max_mtime + 0.1))



def uptodate_fitness(cfg, seeds, filepath):
    """Check that the file at filepath exists and is newer that the results' manifest file."""
    filepath = Path(filepath)
    if not filepath.is_file():
        return False

    max_mtime = float('-inf')
    for seed in seeds:
        fitness_filepath = loadlib.fitness_filepath_from_cfg(cfg, seed)
        max_mtime = max(max_mtime, fitness_filepath.stat().st_mtime)

    return max_mtime < filepath.stat().st_mtime


def set_mtime_fitness(cfg, seeds, filepath):
    """Set the filepath's file access and modification time epsilon newer than the results' manifest file.

    The idea is to ensure that the time comparison of uptodate only occurs with file that were
    generated from this version of the manifest.
    """
    return
    assert len(seeds) > 0
    max_mtime = float('-inf')
    for seed in seeds:
        fitness_filepath = loadlib.fitness_filepath_from_cfg(cfg, seed)
        max_mtime = max(max_mtime, fitness_filepath.stat().st_mtime)

    os.utime(filepath, (max_mtime + 0.1, max_mtime + 0.1))



def uptodate_intersection_done(cfgs, filepath):
    """Check that the file at filepath exists and is newer that the results' manifest file."""
    filepath = Path(filepath)
    if not filepath.is_file():
        return False

    assert len(cfgs) > 0
    seeds = set(loadlib.find_done_cfg(cfgs[0]))
    for cfg in cfgs:
        other_seeds = loadlib.find_done_cfg(cfg)
        seeds = seeds.intersection(other_seeds)

    max_mtime = float('-inf')
    for seed in seeds:
        for cfg in cfgs:
            filepath_manifest = Path(loadlib.manifest_done_path_from_cfg(cfg, seed), 'manifest.yaml')
            max_mtime = max(max_mtime, filepath_manifest.stat().st_mtime)

    return max_mtime < filepath.stat().st_mtime


def set_mtime_intersection_done(cfgs, filepath):
    """Set the filepath's file access and modification time epsilon newer than the results' manifest file.

    The idea is to ensure that the time comparison of uptodate only occurs with file that were
    generated from this version of the manifest.
    """
    return
    assert len(cfgs) > 0
    seeds = set(loadlib.find_done_cfg(cfgs[0]))
    for cfg in cfgs:
        other_seeds = loadlib.find_done_cfg(cfg)
        seeds = seeds.intersection(other_seeds)

    max_mtime = float('-inf')
    for seed in seeds:
        for cfg in cfgs:
            fitness_filepath = loadlib.fitness_filepath_from_cfg(cfg, seed)
            max_mtime = max(max_mtime, fitness_filepath.stat().st_mtime)

    os.utime(filepath, (max_mtime + 0.1, max_mtime + 0.1))
