import subprocess

from . import array_str


def job_status():
    cmd = ['squeue', '--format="%.18i %.9P %.40j %.7K %.8T %.10M %.10l %.5D %.3C %.4m %.20R"', '-hr']
    try:
        squeue_out = subprocess.check_output(cmd, encoding='utf-8')
    except FileNotFoundError:
        return None

    lines = []
    for line in squeue_out.split('\n'):
        if len(line) > 2:
            lines.append(line[1:-1].split())

    n_jobs, status = 0, {}
    for line in lines:
        job_id, partition, version, seeds, state, time, time_limit, nodes, cores, mem, reason = line

        try:
            exp_name, major, minor = _parse_version(version)
            seeds = array_str.str_to_indexes(seeds)
            n_jobs += len(seeds)

            status.setdefault(state, {})
            status[state].setdefault((exp_name, major, minor), set())
            for seed in seeds:
                status[state][(exp_name, major, minor)].add(seed)
        except ValueError:
            pass

    return n_jobs, status

def _parse_version(version):
    exp_name = None
    if '_' in version:
        version, exp_name = version.split('_', 1)
    major = int(version[1:4])
    minor = int(version[6:])
    return exp_name, major, minor

def scheduled_seeds(slurm_state, exp_name, major, minor):
    running_seeds, pending_seeds = [], []
    if slurm_state is not None:
        try:
            running_seeds = slurm_state['RUNNING'][(exp_name, major, minor)]
        except (KeyError, NameError):
            pass
        try:
            pending_seeds = slurm_state['PENDING'][(exp_name, major, minor)]
        except (KeyError, NameError):
            pass

    return running_seeds, pending_seeds
