import sys
import shutil
import traceback
import argparse
from pathlib import Path
import multiprocessing as mp

import evodevo
from evodevo import postlib, Run, utils, paths


def check_seed(cfg, seed):
    filepath_seed = postlib.manifest_path_from_cfg(cfg, seed)

    flag = False
    run = Run.from_results(filepath_seed, verbose=False)
    pop = run.population
    for gen in pop.generations:
        for uid in gen:
            m = pop.repository.member(uid)
            if not (0.0 <= min(m.genotype) and max(m.genotype) <= 1.0):
                print('out of bound genotype: {}, seed {}'.format(m.genotype, seed))
                return

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('exp_name', help='Experiment name', nargs='?', default=evodevo.CURRENT_EXP_NAME)
    parser.add_argument('majors', help="Limit to this major", nargs='?', default=None)
    parser.add_argument('minors', help="Limit to this minor", nargs='?', default=None)
    args = parser.parse_args()

    # exp_name, major, minor = args.experiment.split('.')
    # major, minor = int(major), int(minor)
    try:
        pool = mp.Pool(utils.n_available_cores())

        majors = args.majors
        if majors is not None:
            majors = postlib.str_to_indexes(majors)
        minors = args.minors
        if minors is not None:
            minors = postlib.str_to_indexes(minors)

        for major, minor, _ in postlib.find_cfgs(args.exp_name):
            if ((majors is None or major in majors) and
                (minors is None or minor in minors)):
                cfg_list = []
                print('{}'.format(postlib.version_str(major, minor)))
                cfg = postlib.load_cfg(args.exp_name, major, minor)
                for seed in postlib.find_started_cfg(cfg, with_results=True, done_only=False):
                    cfg_list.append((cfg, seed))
                pool.starmap(check_seed, cfg_list)

    finally:
        pool.close()
        pool.join()
        pool.terminate()
