import sys
import shutil
import traceback
import argparse
from pathlib import Path
import multiprocessing as mp

import evodevo
from evodevo import postlib, Run, utils, paths


def check_seed(cfg, seed, remove=False, directories=(paths.RESULTS_MAIN_DIR,)):
    filepath_seed = postlib.manifest_path_from_cfg(cfg, seed, directories=directories)

    try:
        run = Run.verify(Path(filepath_seed, 'manifest.yaml'))
    except Exception as exc:
        utils.print_error('loading {}'.format(filepath_seed))
        # print(exc, file=sys.stderr)
        traceback.print_exc()
        if remove:
            shutil.rmtree(str(filepath_seed))
            utils.print_info('deleted {}'.format(filepath_seed))
            filepath_seed.mkdir(parents=True, exist_ok=True)

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--exp_name', help='Experiment name', default=evodevo.CURRENT_EXP_NAME)
    parser.add_argument('majors', help="Limit to this major", nargs='?', default=None)
    parser.add_argument('minors', help="Limit to this minor", nargs='?', default=None)
    parser.add_argument('--remove', '-r', help="remove the compromised runs.",
                        default=False, action='store_true')
    args = parser.parse_args()

    # exp_name, major, minor = args.experiment.split('.')
    # major, minor = int(major), int(minor)
    try:
        pool = mp.Pool(utils.n_available_cores())

        majors = args.majors
        if majors is not None:
            majors = postlib.str_to_indexes(majors)
        minors = args.minors
        if minors is not None:
            minors = postlib.str_to_indexes(minors)

        for major, minor, _ in postlib.find_cfgs(args.exp_name):
            if ((majors is None or major in majors) and
                (minors is None or minor in minors)):
                cfg_list = []
                print('{}'.format(postlib.version_str(major, minor)))
                cfg = postlib.load_cfg(args.exp_name, major, minor)
                for seed in postlib.find_started_cfg(cfg, with_results=True, done_only=False, directories=[paths.RESULTS_MAIN_DIR]):
                    cfg_list.append((cfg, seed, args.remove, (paths.RESULTS_MAIN_DIR,)))
                pool.starmap(check_seed, cfg_list)

    finally:
        pool.close()
        pool.join()
        pool.terminate()
