import os
import sys
import glob
import time
import socket
import argparse
from pathlib import Path

import evodevo
from evodevo import paths


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('exp_name', help="name of the experiment to process", nargs='?', default=evodevo.CURRENT_EXP_NAME)
    parser.add_argument('--results_dir', '-r', help="results dir", default=paths.RESULTS_MAIN_DIR)
    parser.add_argument('--pretend', '-p', help="only print what the script would do.",
                        default=False, action='store_true')
    parser.add_argument('--force', '-f', default=False, action='store_true',
                        help="required when not on the cluster, to avoid accidental loss of data.")
    args = parser.parse_args()

    if not args.force and not socket.gethostname().startswith('deigo'):
        print('Not on cluster. Use --force to delete files.')
        sys.exit(1)

    if args.exp_name is None:
        exp_names = ['devoevo_draft_delay']
    else:
        exp_names = [args.exp_name]

    def delete(filename):
        if args.pretend:
            print('would delete {}'.format(filename))
        else:
            print('deleting {}'.format(filename))
            os.remove(filename)

    for exp_name in exp_names:
        directory = Path(args.results_dir, exp_name)
        for root, dirs, files in os.walk(directory, topdown=False):
            #dirs.sort()
            if Path(root).name.startswith('seed'):
                done_path = Path(root, 'done')
                # done file exist and is more than three hours old.
                if done_path.is_file() and time.time() - done_path.stat().st_mtime > 3 * 3600.0:
                    for ext in ['*.yaml', '*.msgpack.xz', 'lock', '*.err', '*.out']:
                        for filename in sorted(glob.glob(str(Path(root, ext)))):
                            delete(filename)
