#!/usr/bin/env python

""" Developmental Learning Experiment

A population of controller starts will a small body that grows with each passing generation.
"""
import os
import sys
import argparse

import numpy as np
import evodevo
from evodevo import utils
from evodevo import paths


if __name__ == '__main__':
    # parsing external arguments
    parser = argparse.ArgumentParser()
    parser.add_argument('initial_pop_config', help="configuration filepath")
    parser.add_argument('--progress', help="show progress bar", action='store_true', default=False)
    args = parser.parse_args()

    ip_params = evodevo.load_config(args.initial_pop_config)
    cfg_filepath = os.path.join(os.path.dirname(args.initial_pop_config), ip_params['cfg_filename'])

    run_params = evodevo.load_config(cfg_filepath)
    run_params['meta']['seed'] = ip_params['seed']
    run_params['meta']['save_filepath'] = None
    run_params['population']['n_mutant'] = ip_params['n_candidates']

    # create the initial population
    run = evodevo.Run(run_params, resume=False)
    run.population.manual_generation()
    gen = []
    for _ in range(ip_params['n_candidates']):
        gen.append(run.population.manual_member(genotype=None, params=run_params['member']))

    run.population.evaluate_generation(gen)
    initial_pop = sorted(gen, key=lambda x: x.fitness, reverse=True)[:ip_params['n_initial_pop']]

    initial_genotypes = np.array([m.genotype for m in initial_pop])
    pop_path = os.path.join(paths.DATA_DIR, ip_params['filename'])
    os.makedirs(os.path.dirname(pop_path), exist_ok=True)
    with open(pop_path, 'wb') as fd:
        np.save(fd, initial_genotypes)

    print('saved initial pop in {} ({} genotypes)'.format(pop_path, ip_params['n_initial_pop']))
    run.population.close()
