import sys
import argparse

# import colored
# from colored import stylize
from termcolor import colored, cprint
import numpy as np

import evodevo
from evodevo import postlib


def fmt(member):
    s = '{:7.1f}'.format(member.fitness)
    if member.survives and not member.parent:
        return colored(s, 'red', attrs=[])
    # if member.parent and member.survives:
    #     return colored(s, 'red',  attrs=['underline'])
    # #     # return stylize(s, colored.fg("red")   + colored.attr("bold"))
    if member.parent:
        return colored(s, 'green')
        # return stylize(s, colored.fg("black") + colored.attr("bold"))
    #     # return stylize(s, colored.fg("red")   + colored.attr("reset"))
    else:
        return colored(s, 'white', attrs=[])
        # return stylize(s, colored.fg("black") + colored.attr("reset"))

def score_sort(members, previous_gen):
    if previous_gen is None:
        return 0, 0

    gen = sorted(members, key=lambda m: m.fitness, reverse=True)
    previous_gen = sorted(previous_gen, key=lambda m: m.fitness, reverse=True)
    for i, m in enumerate(previous_gen):
        m.rank = i + 1
    previous_map = {m.uid: m for m in previous_gen}

    parents, children = [], []
    for m in gen:
        parent = previous_map[m.puid]
        m.p_rank = parent.rank
        if parent.genotype == m.genotype:
            parents.append(m)
        else:
            children.append(m)

    # print(len(parents), len(children))
    def inversions(ms):
        _score = 0
        for i, m in enumerate(ms):
            for m2 in ms[i+1:]:
                # print(i, m.fitness, m.p_rank, m2.fitness, m2.p_rank)
                if m.p_rank > m2.p_rank:
                    _score += 1
        return _score

    return inversions(parents), inversions(children)

def score_stability_parent(members, previous_gen):
    if previous_gen is None:
        return 0.0

    gen = sorted(members, key=lambda m: m.fitness, reverse=True)
    previous_gen = sorted(previous_gen, key=lambda m: m.fitness, reverse=True)
    for i, m in enumerate(previous_gen):
        m.rank = i + 1
    previous_map = {m.uid: m for m in previous_gen}

    fitness, fitness_diff = [], []
    for m in gen:
        parent = previous_map[m.puid]
        if parent.genotype == m.genotype:
            fitness.append(abs(parent.fitness))
            fitness.append(abs(m.fitness))
            fitness_diff.append(abs(parent.fitness - m.fitness))

    return np.mean(fitness_diff)/np.mean(fitness)



if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('exp_name', help='Experiment name')
    parser.add_argument('major', help="major", type=int)
    parser.add_argument('minor', help="minor", type=int)
    parser.add_argument('seed', help="seed", type=int)
    args = parser.parse_args()

    run = postlib.load_results(args.exp_name, args.major, args.minor, args.seed)
    pop = run.population

    previous_gen = None
    for i, gen in enumerate(pop.generations):
        members = [pop.repository.member(uid) for uid in gen]
        members = sorted(members, key=lambda m: m.fitness)
        for m in members:
            m.parent = False
            m.survives = False

        next_gen = None
        if i < len(pop.generations) - 1:
            next_gen_genotypes = set([tuple(pop.repository.member(uid).genotype) for uid in pop.generations[i+1]])
            for m in members:
                g = tuple(m.genotype)
                if g in next_gen_genotypes:
                    m.survives = True

        if previous_gen is not None:
            mp_genotypes = set([tuple(m_p.genotype) for m_p in previous_gen])
            for m in members:
                g = tuple(m.genotype)
                if g in mp_genotypes:
                    m.parent = True

        parent_score, children_score = score_sort(members, previous_gen)
        stability = score_stability_parent(members, previous_gen)
        print('{:04d}: {} | {: 2d} {: 3d} | {: 7.4f}'.format(i, ' '.join(fmt(m) for m in members),
                                                             parent_score, children_score, stability))
        # print('      {}'.format(' '.join('{: 7d}'.format(m.uid) for m in members)))

        previous_gen = members
