import os
import numbers
from pathlib import Path

import numpy as np

from evodevo import postlib, paths, devo
from evodevo.postlib import graph


color_list = ['#332288', '#88ccee', '#44aa99', '#117733', '#999933', '#ddcc77']
#color_list = ['#045a8d', '#2b8cbe', '#74a9cf', '#a6bddb', '#d0d1e6', '#f1eef6']


def splice(Y):
    if isinstance(Y[0], numbers.Number):
        return [Y]
    else:
        return list(zip(*Y))

def dev_traj(exp_name, major, minor, dev_name, T=4000, **kwargs):
    cfg = postlib.load_cfg(exp_name, major, minor)
    dev_params = cfg['population']['dev_vars'][dev_name]
    factor = 20000 if dev_name == 'muscle_stiff' else 1.0

    dev_gen = devo.TrajectoryGenerator(dev_params)
    x = range(T)
    return x, [dev_gen(t)/factor for t in x]

def draw_dev_traj(major, minor, dev_name, color, line_size=4, scatter_size=10,
                  y_range=[0.8, 1.2], y_ticker=[0, 1], height=200, font_size='20pt'):
    exp_name = 'wobbling'
    width = 600

    x, y = dev_traj(exp_name, major, minor, dev_name, T=4000)
    title = '{}.{}_{}'.format(major, minor, dev_name)
    g = graph.Graph(title=title, width=width, height=height,
                    x_axis_location='above', y_range=y_range)
    g.fig.axis.major_label_text_font = 'avenir next'
    g.fig.axis.major_label_text_font_size = font_size
    color = color if color is not None else color_list[1]
    g.fig.line(x, y, alpha=0.75, line_color=color, line_width=line_size)
    g.fig.scatter([0, 2000, 3999], [y[0], y[2000], y[3999]], alpha=0.75,
                fill_color=color, line_color=None, size=scatter_size)
    g.fig.yaxis.ticker = y_ticker
    g.fig.xaxis.ticker = [0, 2000, 4000]

    fig_filepath = Path(paths.FIGURES_DIR, exp_name, 'devgraph', title + '.pdf')
    graph.save_fig(g.fig, fig_filepath, scale_factor=1.0)


if __name__ == '__main__':
    for major, minor, dev_name, color in [
                                        #   (2, 4, 'node_mass', None),
                                          (9, 6, 'node_mass', None)
                                         ]:
        draw_dev_traj(major, minor, dev_name, color, line_size=1, scatter_size=5,
                      y_range=(0.3, 1.7), y_ticker=(0.5, 1, 1.5), height=150,
                      font_size='12pt')

    for major, minor, dev_name, color in [(2, 8, 'node_mass', '#e457a0'),
                                          (3, 8, 'node_mass', '#d783ff'),
                                          (4, 8, 'node_mass', '#942193'),
                                         ]:
        draw_dev_traj(major, minor, dev_name, color, line_size=2, scatter_size=5,
                      y_range=(0.8, 1.2), y_ticker=(0.9, 1), height=200)
