import sys
import pickle
from operator import ge

sys.setrecursionlimit(10000)

import evodevo.evo.analysis
from evodevo import postlib, utils



def genealogy_distance(major, minor, seeds, verbose=False):
    genealogy_dist_2000 = {}
    genealogy_dist_4000 = {}

    for seed in seeds:
        if verbose:
            print(f'v{major}.{minor} seed {seed}')
        genealogy_dist_2000.setdefault((major, minor), [])
        genealogy_dist_4000.setdefault((major, minor), [])

        run = postlib.load_results('wobbling', major, minor, seed)
        pop = run.population

        def elite_member(pop, gen):
            members = [pop.repository.member(uid) for uid in pop.generations[gen]]
            members = sorted(members, key=lambda x: x.fitness)
            return members[-1]

        m0    = elite_member(pop, 0)
        # print(m0.body_size(0))
        m2000 = elite_member(pop, 2000)
        m3999 = elite_member(pop, 3999)

        # if verbose:
        #     print('genotype:  m0    <-> m2000: {:.2f}'.format(evodevo.evo.analysis.genotype_distance(m0, m2000)))
        #     print('genotype:  m2000 <-> m3999: {:.2f}'.format(evodevo.evo.analysis.genotype_distance(m2000, m3999)))

        members, distances = evodevo.evo.analysis.genealogy_distance(pop, uids=[m2000.uid], window=[0, 2000])
        for m, d in zip(members, distances):
            if m.uid == m2000.uid:
                genealogy_dist_2000[(major, minor)].append(d)
                if verbose:
                    print('genealogy: m0    <-> m2000: {:.2f}'.format(d))

        members, distances = evodevo.evo.analysis.genealogy_distance(pop, uids=[m3999.uid], window=[2000, 4000])
        for m, d in zip(members, distances):
            if m.uid == m3999.uid:
                genealogy_dist_4000[(major, minor)].append(d)
                if verbose:
                    print('genealogy: m2000 <-> m3999: {:.2f}'.format(d))
    if verbose:
        print('')

    return genealogy_dist_2000, genealogy_dist_4000


def genealogy_distance_mean(major, minor):
        filepath = evodevo.paths.POST_DIR / f'gen_dist_v{major}.{minor}.pickle'
        if filepath.exists():
            with open(filepath, 'rb') as fd:
                genealogy_dist_2000, genealogy_dist_4000 = pickle.load(fd)
        else:
            genealogy_dist_2000, genealogy_dist_4000 = genealogy_distance(major, minor, list(range(1, 101)), verbose=False)
            filepath.parent.mkdir(parents=True, exist_ok=True)
            with open(filepath, 'wb') as fd:
                pickle.dump((genealogy_dist_2000, genealogy_dist_4000), fd)

        print(f'v{major}.{minor}')
        metrics = postlib.metrics.metrics_acute(genealogy_dist_2000[(major, minor)], include_data=False,
                                                confidence_median=None, confidence_mean=0.99)
        print('mean genealogy: m0    <-> m2000: {:.2f}+{:.2f}-{:.2f}'.format(metrics['mean'], metrics['mean_ci_high'] - metrics['mean'], - metrics['mean_ci_low'] + metrics['mean']))
        metrics = postlib.metrics.metrics_acute(genealogy_dist_4000[(major, minor)], include_data=False,
                                                confidence_median=None, confidence_mean=0.99)
        print('mean genealogy: m2000 <-> m3999: {:.2f}+{:.2f}-{:.2f}'.format(metrics['mean'], metrics['mean_ci_high'] - metrics['mean'], - metrics['mean_ci_low'] + metrics['mean']))



if __name__ == '__main__':
    # seed 10 for figure 4
    genealogy_distance(8, 1, [10], verbose=True)
    genealogy_distance(8, 2, [10], verbose=True)

    for major, minor in [(8, 1), (8, 2)]:
        genealogy_distance_mean(major, minor)
