# Code for "Morphological Wobbling Can Help Robots Learn"

This code and the associated data are meant to allow the reproduction of the results of the article:
> Fabien C. Y. Benureau, Jun Tani; Morphological Wobbling Can Help Robots Learn. (submitted to ICDL 2022)

## Installation

To install the Springs physics engine, in the `../springs/` folder:
```bash
pip install wheel cython
pip install -e .
```

Then, in this repository root directory:
```bash
pip install -e .
```

To test that everything works, you can run a simulation on a random genotype from one of the conditions. For instance, to display a simulation, in `experiments`:
```bash
python show.py random wobbling/v002_mass/v2_01_sinusoid5_1.0_0.1.yaml
```

## Issues

This code is for archival and investigatory purposes. It has been lightly tested within the needs of its experimental purposes. The code was run on Linux (Ubuntu 20.04, Arch Linux 2021) and MacOS (Intel 2020, Arm M1 2021). You may run into issues installing or running, especially as time from publication passes. Do not hesitate to reach me at fabien@benureau.com or on GitHub github.com/benureau/wobbling2022 if that happens.


## Computing Pipeline

To get from code to figure the computing pipeline is organized as follows:
1. run simulations (probably on a computing cluster), producing ~11Gb of logs.
2. post-process the results, reducing it to ~10Mb of data
3. produce figures from post-processed results.

With the code and data provided here, you can run independently either of those steps. Step 1 is by far the longest, as detailed below. You can download the results of Step 1 from Zenodo at http://doi.org/10.5281/zenodo.6513360. They should be unpacked in the directory about the root of this repository:
```bash
cd ..
tar xfv results_archives.tar
```
This will create a `results_archives/` directory next to the `code`, `post`, and `results` directory.



## Running Simulations

The results of the paper are based on a large number of simulations. They were only possible using a university cluster. Even without a cluster, you can run an individual simulation with:
```bash
python run_exp.py wobbling/v002_mass/v2_01_sinusoid5_1.0_0.1.yaml --seed 0
```
(published simulations use seeds 1 to 100, both included; in this example, seed 0 is used to not interfere with existing results.)

A simulation takes of the order of one hour for each seed on a 20-core CPU. There are 100 seeds per experimental condition, and 51 conditions. On a 20-core CPU, computing everything will take of the order of 7 months.

Our cluster system used [slurm](https://slurm.schedmd.com/documentation.html), and so the scripts are adapted to it. It is also adapted to the specificity of our cluster configuration, user limits, etc. To run the simulation on your cluster,
adapts the file `slurm_batch.py` as appropriate in the folder `experiments/`. Then, to run the control condition for instance (one condition, 100 seeds):
```bash
python slurm_batch.py wobbling/v001_control/*
```

Note that the script will detect which simulations are already finished (results files are present and complete) and will not start those again. It will also attempt to detect running simulations and avoid launching duplicates on slurm clusters.

You can check which simulations are finished/running/still need to be launched with:
```bash
python check_cfgs.py --finished
```
The code detecting running simulations is quite fragile. To disable it:
```bash
python check_cfgs.py --finished --nosqueue
```
More generally, `python check_cfgs.py --help` will display some documentation.


## Postprocessing

The simulation produces at a total of ~11Gb of data. We post-process that data to first create the reduced dataset that will be useful to generate the figures. This process can take a long time. It will adjust to the number of CPU cores available on the machine, so a high-core count machine is recommended for a fast computation.

In `experiments/`:
```bash
python post.py wobbling/post_paper.yaml
python wobbling/figures/gen_dist_pca.py
```


## Article Assets

### To Regenerate Figures

Many figures are generated using [bokeh 2.4](https://docs.bokeh.org/en/latest/index.html), which requires some setup (as of 2021) to save figures to disk. See https://docs.bokeh.org/en/latest/docs/user_guide/export.html.

All the commands below can be launched together from the `generate_figures.sh` (to execute from the `experiments/` folder).

For boxplot plots (in Figures 3), in `experiments/wobbling/figures`:
```bash
python fig_boxplot.py
```

For the developmental trajectories of Figure 2, in `experiments/wobbling/figures`:
```bash
python fig_dev_schedule.py
```

For Figures 2.A.a-b, in `experiments/wobbling/figures`:
```bash
python fig_scatter.py
```

For all other figures, in `experiments/`:
```bash
python post.py wobbling/figures/graphs_paper.yaml
```


### To Recompute Numerical Figures

For the genealogy distance measurements of Figure 4.A in `experiments/wobbling/figures`:
```bash
python gen_dist_pca.py
```

For the significance calculation of Figures 3.c, 4.a-c, S5.b, 6.a-b, and 7, in `experiments/wobbling/figures`:
```bash
python compute_significance.py
```


### To Re-Record Movies

For the script used to generate movie S1, see `movies.sh` in `experiments/` (the script is pretty unoptimized and takes a long time to run).
